#! /usr/bin/env python

## Copyright 2001 by LivingLogic AG, Bayreuth, Germany.
## Copyright 2001 by Walter Drwald
##
## All Rights Reserved
##
## Permission to use, copy, modify, and distribute this software and its documentation
## for any purpose and without fee is hereby granted, provided that the above copyright
## notice appears in all copies and that both that copyright notice and this permission
## notice appear in supporting documentation, and that the name of LivingLogic AG or
## the author not be used in advertising or publicity pertaining to distribution of the
## software without specific, written prior permission.
##
## LIVINGLOGIC AG AND THE AUTHOR DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
## INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
## LIVING LOGIC AG OR THE AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
## DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
## IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
## IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

r"""
<par>A module that provides some tools for file i/o related stuff.</par>

<par>The first is a class <code>Filename</code>, that provides several
methods (e.g. a method <code>open</code>, that automatically creates
any non exiting directories.</par>

<par>The second is a class named <code>SafeFile</code>, that encapsulates
a normal file. Nothing will be written to the file with the specified
name. All writes go to a temporary file instead. Only when the
method <code>close</code> is called, will the temporary file be closed
and the file renamed to the original name.</par>

<par>The third tool is a class <code>LogFile</code>. Writing to a log
file prepends a timestamp to each line.</par>
"""

__version__ = tuple(map(int,"$Revision: 1.12 $"[11:-2].split(".")))
# $Source: /data/cvsroot/LivingLogic/fileutils/fileutils.py,v $

import sys, os, os.path, time, UserString

from mx import DateTime

class Filename(UserString.UserString):
	def __init__(self, name):
		if isinstance(name, Filename):
			self._fullname = name._fullname
			name = str(name)
		else:
			name = os.path.normpath(name)
			if name == "":
				name = os.curdir
			self._fullname = os.path.abspath(os.path.expanduser(name))
		UserString.UserString.__init__(self, name)

	def clone(self):
		return self.__class__(self)

	def __cmp__(self, other):
		res = cmp(self.__class__.__name__, other.__class__.__name__)
		if not res:
			res = cmp(self._fullname, other._fullname)
		return res

	def __hash__(self):
		return hash(self._fullname)

	def fullname(self):
		return self._fullname

	def ext(self):
		return os.path.splitext(self._fullname)[1]

	def open(self, mode="rb", buffering=-1):
		"""
		Works like the builtin function open, but is able to create
		non existing directories.
		"""
		try:
			return open(self._fullname, mode, buffering)
		except IOError, ex:
			if "w" in mode:
				if ex[0] != 2: # didn't work for some other reason than a non existing directory
					raise
				(splitpath, splitname) = os.path.split(self._fullname)
				if splitpath!="": # we really have a parent
					os.makedirs(splitpath)
					return open(self._fullname, mode, buffering)
			raise # we don't have a directory to make (or open was for reading), so pass the error on

	def remove(self):
		os.remove(self._fullname)

	def rename(self, other):
		os.rename(self._fullname, other._fullname)

	def getmtime(self):
		timestamp = os.path.getmtime(self._fullname)
		tuple = time.localtime(timestamp)[:6]
		return DateTime.DateTime(*tuple)

	def mkdir(self):
		os.makedirs(self._fullname)

	def mkparentdir(self):
		(parent, dummy) = os.path.split(self._fullname)
		if parent != self._fullname:
			os.makedirs(parent)

	def files(self):
		files = {}
		os.path.walk(str(self), self.__walk, files)
		files = files.keys()
		files.sort()
		return [ Filename(file) for file in files ]

	def __walk(self, files, dirname, filenames):
		try:
			del files[dirname] # don't collect directories
		except KeyError:
			pass
		for filename in filenames:
			files[os.path.join(dirname, filename)] = None

	def __add__(self, other):
		return self.__class__(os.path.join(str(self), str(other)))

	def __radd__(self, other):
		return self.__class__(os.path.join(str(other), str(self)))

	def __repr__(self):
		return "%s(%s)" % (self.__class__.__name__, UserString.UserString.__repr__(self))

class SafeFile:
	"""
	encapsulates a normal file.
	When the file will be opened, all writes go to
	a temporary file. After all writes are finished
	to file will be renamed to its final name.
	"""

	def __init__(self, name, mode="w", buffering=0):
		self.name = Filename(name)
		self.tmpname = self.name + ("_%d.tmp" % os.getpid())
		self.mode = mode
		self.buffering = buffering
		self.file = None

	def write(self, *alltexts):
		self.__open()
		for text in alltexts:
			self.file.write(text)

	def writelines(self, *alllines):
		self.__open()
		for lines in alllines:
			self.file.writelines(lines)

	def __open(self):
		if self.file is None:
			self.file = self.tmpname.open(self.mode, self.buffering)

	def close(self):
		"""
		This method can be called by the user when all writes to the file
		are done. The file will be closed and moved to the final destination.
		"""
		if self.file is not None:
			self.file.close()
			self.tmpname.rename(self.name)
			self.file = None

class LogFile:
	"""
	A log file. All lines written to the file will be prepended with a time stamp.
	"""
	def __init__(self, name, mode="a", buffering=1, encoding="iso-8859-1"):
		"""
		__init__(filename, mode="a", buffering=1, encoding="iso-8859-1") -- construct a Log file (will be opened on the first write)
		
		filename -- the filename (this may include "~/" or "~spam/" as os.path.expanduser()
		  will be called.)
		encoding -- the encoding to use for the strings written to the file
		mode -- The mode for opening the file (should be "w" or "a")
		buffering -- the buffering for the file (0==unbuffered, 1==line, other==buffersize)
		"""
		self.startTime = time.time()
		self.name = name
		self.mode = mode
		self.buffering = buffering
		self.encoding = encoding
		self.file = None

	def formatTime(self, timestamp):
		"""
		format the timestamp into a string.
		"""
		return time.strftime("%d/%b/%Y %H:%M:%S", time.localtime(timestamp))

	def __open(self):
		if self.file is None:
			self.file = Filename(self.name).open(self.mode, self.buffering)

	def write(self, *texts):
		"""
		self.log(*texts) -- write texts to the log file
		"""
		now = time.time()
		pid = os.getpid()
		self.__open()
		for text in texts:
			lines = text.encode(self.encoding).split("\n")
			if not len(lines[-1]):
				del lines[-1]
			lines = [ "[pid=%d][%s]=[t+%.2fsec] %s\n" % (pid, self.formatTime(now), now-self.startTime, line) for line in lines ]
			self.file.write("".join(lines))

